;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; File:         timer.cl
; Description:  The timer function for Gabriel's test suite.
; Author:       Robert Kessler, Will Galway and Stan Shebs
; Created:      05-Mar-84
; Modified:     16-Dec-85 (Stan Shebs)
; Mode:         PCLS
; Package:      User
; Status:       Experimental
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Invoke this function to run a benchmark.  The first argument is a string
;;; identifying the benchmark, while the second is the form to be evaluated.

(defun run-benchmark (name form)
  (let ((fname (gentemp)))
    (eval `(defun ,fname () ,(build-timer-function name form)))
    (compile fname)
    (apply fname nil)))

;;; The following function builds the body of the timer function

(defun build-timer-function (string form)
  (setq string (string string))
  `(progn
     ; Make all tests start with a reclaim so that we start out with a
     ; "clean" stack.
     (psl:reclaim)

     (format t "~%--------------------------------------------------------~%")
     (format t "~A~%" ,string)
     (format t "Timing performed on ~A ~A running ~A ~A (~A) on ~A.~%"
	     (machine-type) (machine-version)
	     (software-type) (software-version)
	     (machine-instance)
	     (date-and-time))

     ((psl:lambda (t1 t2 t3 gt lb la)
        ,form
        (setq t1 (- (run-time) t1)
          t2 (- (elapsed-time) t2)
          t3 (- (wholine-time) t3)
          gt (- (gc-time) gt)
          la (load-average))
        (format t "~%........................................~%")
        (format t "Cpu (- GC) Time = ~A secs~%" 
		(/ (float (- t1 gt)) 1000.))
        (format t "Elapsed Time = ~A secs~%" (/ (float t2) 1.0))
        (format t "GC Time = ~A secs~%" (/ (float gt) 1000.0))
        (format t "Load Average Before  = ~A~%" lb)
        (format t "Load Average After   = ~A~%" la)
        (format t "Average Load Average = ~A~%" (/ (+ la lb) 2.0))
        (format t "--------------------------------------------------------~%")
        )
       (run-time) (elapsed-time) (wholine-time) (gc-time) (load-average) 0)))

;;; Return date and time as a string

(defun date-and-time () (psl:current-date-time))

(defun run-time () (psl:time))

;;; Wholine time.  Don't know what this is.

(defun wholine-time () 0)

;;; Return elapsed time in seconds.  A real kludge, have to pick it up from
;;; the date and time string.

(defun elapsed-time ()
  (let* ((dt (date-and-time))
	 ;; format is "05-mar-84 09:49:50"
         (hour (read-from-string (psl:sub dt 10 1)))
	 (min (read-from-string (psl:sub dt 13 1)))
	 (sec (read-from-string (psl:sub dt 16 1))))
    (+ sec (* 60 (+ min (* 60 hour))))))

(defun load-average () 0)

(defun gc-time () psl:gctime*)
